@@ -98,13 +98,12 @@ jni_threads_unlock()
 
 static gpointer jni_loop_callback(void *data)
 {
-  char *message;
-  message = (char *)data;
   if (!g_main_loop_is_running((GMainLoop *)data))
     g_main_loop_run((GMainLoop *)data);
-  else {
-      if (jaw_debug)
-    printf("Running %s already\n", message);
+  else
+  {
+    if (jaw_debug)
+      printf("Running JNI already\n");
   }
   return 0;
 }
@@ -122,6 +121,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_initNativeLibrary(JNIEnv *jniEnv
     return JNI_TRUE;
   // Java app with GTK Look And Feel will load gail
   // Set NO_GAIL to "1" to prevent gail from executing
+
   g_setenv("NO_GAIL", "1", TRUE);
 
   // Disable ATK Bridge temporarily to aoid the loading
@@ -147,6 +147,7 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_loadAtkBridge(JNIEnv *jniEnv,
   char * message;
   message = "JNI main loop";
   err = NULL;
+  jobject obj;
 
   jaw_initialized = jaw_accessibility_init();
   if (jaw_debug)
@@ -162,6 +163,9 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_loadAtkBridge(JNIEnv *jniEnv,
       g_error_free (err);
     }
   }
+  lock = (*jniEnv)->NewGlobalRef(jniEnv, obj);
+  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
+  gdk_threads_init();
 }
 
 enum _SignalType {
@@ -222,18 +226,19 @@ free_callback_para (CallbackPara *para)
 
   if (para->global_ac == NULL)
   {
-    fprintf(stderr,"\n *** free_callback_para: para->global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("free_callback_para: para->global_ac == NULL");
     free_callback_para(para);
     return;
   }
 
-	(*jniEnv)->DeleteGlobalRef(jniEnv, para->global_ac);
+  (*jniEnv)->DeleteGlobalRef(jniEnv, para->global_ac);
 
-	if (para->args) {
-		(*jniEnv)->DeleteGlobalRef(jniEnv, para->args);
-	}
+  if (para->args) {
+    (*jniEnv)->DeleteGlobalRef(jniEnv, para->args);
+  }
 
-	g_free(para);
+  g_free(para);
 }
 
 static gboolean
@@ -245,20 +250,23 @@ focus_notify_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** focus_notify_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("\nfocus_notify_handler: env == NULL\n");
     free_callback_para(para);
     return FALSE;
   }
   if (global_ac == NULL)
   {
-    g_warning("\n *** focus_notify_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("\nfocus_notify_handler: global_ac == NULL\n");
     free_callback_para(para);
     return FALSE;
   }
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** focus_notify_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("\nfocus_notify_handler: jaw_impl == NULL\n");
     free_callback_para(para);
     return FALSE;
   }
@@ -278,10 +286,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_focusNotify(JNIEnv *jniEnv,
                                                             jclass jClass,
                                                             jobject jAccContext)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(focus_notify_handler, para);
 }
 
@@ -295,20 +301,26 @@ window_open_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
+    if (jaw_debug)
+      fprintf(stderr,"\n *** window_open_handler: jniEnv == NULL *** \n");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    fprintf(stderr,"\n *** window_open_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      fprintf(stderr,"\n *** window_open_handler: global_ac == NULL *** \n");
     free_callback_para(para);
     return FALSE;
   }
 
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
-    g_warning("\n *** window_open_handler: jaw_impl == NULL *** \n");
+  {
+    if (jaw_debug)
+      g_warning("window_open_handler: jaw_impl == NULL");
+  }
   AtkObject* atk_obj = ATK_OBJECT(jaw_impl);
 
   if (!g_strcmp0(atk_role_get_name(atk_object_get_role(atk_obj)),
@@ -351,11 +363,9 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowOpen(JNIEnv *jniEnv,
                                                            jboolean jIsToplevel)
 {
 
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   para->is_toplevel = (jIsToplevel == JNI_TRUE) ? TRUE : FALSE;
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
   g_idle_add(window_open_handler, para);
 }
 
@@ -369,21 +379,24 @@ window_close_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_close_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_close_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_close_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_close_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
   JawImpl *jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_close_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_close_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -427,12 +440,9 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowClose(JNIEnv *jniEnv,
                                                             jobject jAccContext,
                                                             jboolean jIsToplevel)
 {
-
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   para->is_toplevel = (jIsToplevel == JNI_TRUE) ? TRUE : FALSE;
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
   g_idle_add(window_close_handler, para);
 }
 
@@ -445,21 +455,24 @@ window_minimize_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_minimize_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_minimize_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_minimize_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_minimize_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_minimize_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_minimize_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -477,10 +490,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMinimize(JNIEnv *jniEnv,
                                                                jclass jClass,
                                                                jobject jAccContext)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(window_minimize_handler, para);
 }
 
@@ -493,14 +504,16 @@ window_maximize_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_maximize_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_maximize_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_maximize_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_maximize_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -508,7 +521,8 @@ window_maximize_handler (gpointer p)
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_maximize_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_maximize_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -525,10 +539,8 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowMaximize(JN
                                                                               jclass jClass,
                                                                               jobject jAccContext)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac );
   g_idle_add(window_maximize_handler, para);
 }
 
@@ -541,14 +553,16 @@ window_restore_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_restore_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_restore_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_restore_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_restore_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -556,7 +570,8 @@ window_restore_handler (gpointer p)
   JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_restore_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_restore_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -574,10 +589,8 @@ JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowRestore(JNI
                                                                              jobject jAccContext)
 {
 
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(window_restore_handler, para);
 }
 
@@ -590,21 +603,24 @@ window_activate_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_activate_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_activate_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_activate_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_activate_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
-  JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
+  JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_activate_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_activate_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -618,12 +634,11 @@ window_activate_handler (gpointer p)
 }
 
 JNIEXPORT void JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowActivate(JNIEnv *jniEnv,
-                                                                              jclass jClass, jobject jAccContext) {
+                                                                              jclass jClass,
+                                                                              jobject jAccContext) {
 
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
-  CallbackPara *para = alloc_callback_para(lock);
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(window_activate_handler, para);
 }
 
@@ -636,21 +651,24 @@ window_deactivate_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_deactivate_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_deactivate_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_deactivate_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_deactivate_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_deactivate_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_deactivate_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -669,11 +687,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowDeactivate(JNIEnv *jniEnv,
                                                                  jobject jAccContext)
 {
 
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
-
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(window_deactivate_handler, para);
 }
 
@@ -686,14 +701,16 @@ window_state_change_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** window_state_change_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_state_change_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** window_state_change_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_state_change_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -701,7 +718,8 @@ window_state_change_handler (gpointer p)
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** window_state_change_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("window_state_change_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -720,10 +738,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_windowStateChange(JNIEnv *jniEnv
                                                                   jobject jAccContext)
 {
 
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(window_state_change_handler, para);
 }
 
@@ -774,7 +790,8 @@ signal_emit_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** signal_emit_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("signal_emit_handler: jniEnv == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -783,7 +800,8 @@ signal_emit_handler (gpointer p)
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** signal_emit_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("signal_emit_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -792,7 +810,8 @@ signal_emit_handler (gpointer p)
 
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** signal_emit_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("signal_emit_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -803,9 +822,8 @@ signal_emit_handler (gpointer p)
   {
     case Sig_Text_Caret_Moved:
     {
-      gint cursor_pos = get_int_value(
-      jniEnv,
-      (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
+      gint cursor_pos = get_int_value(jniEnv,
+                                      (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
       g_signal_emit_by_name(atk_obj, "text_caret_moved", cursor_pos);
       break;
     }
@@ -841,7 +859,8 @@ signal_emit_handler (gpointer p)
       JawImpl *child_impl = jaw_impl_get_instance(jniEnv, child_ac);
       if (child_impl == NULL)
       {
-        g_warning("\n *** signal_emit_handler: child_impl == NULL *** \n");
+        if (jaw_debug)
+          g_warning("signal_emit_handler: child_impl == NULL");
         free_callback_para(para);
         return FALSE;
       }
@@ -878,10 +897,11 @@ signal_emit_handler (gpointer p)
       case Sig_Object_Active_Descendant_Changed:
       {
       jobject child_ac = (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0);
-      JawImpl *child_impl = jaw_impl_find_instance(jniEnv, child_ac);
+      JawImpl *child_impl = jaw_impl_get_instance(jniEnv, child_ac);
       if (child_impl == NULL)
       {
-        g_warning("\n *** signal_emit_handler: child_impl == NULL *** \n");
+        if (jaw_debug)
+          g_warning("signal_emit_handler: child_impl == NULL");
         free_callback_para(para);
         return FALSE;
       }
@@ -910,7 +930,7 @@ signal_emit_handler (gpointer p)
     case Sig_Object_Property_Change_Accessible_Actions:
     {
       gint oldValue = get_int_value(jniEnv,
-                                  (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
+                                    (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
       gint newValue = get_int_value(jniEnv,
                                     (*jniEnv)->GetObjectArrayElement(jniEnv, args, 1));
       AtkPropertyValues values = { NULL };
@@ -922,15 +942,13 @@ signal_emit_handler (gpointer p)
       g_value_set_int(&values.old_value, oldValue);
       if (jaw_debug)
         printf ("%d\n", g_value_get_int(&values.old_value));
-      g_value_unset(&values.old_value);
 
       g_assert (!G_VALUE_HOLDS_INT (&values.new_value));
       g_value_init(&values.new_value, G_TYPE_INT);
-      g_assert (G_VALUE_HOLDS_INT (&values.old_value));
+      g_assert (G_VALUE_HOLDS_INT (&values.new_value));
       g_value_set_int(&values.new_value, newValue);
       if (jaw_debug)
         printf ("%d\n", g_value_get_int(&values.new_value));
-      g_value_unset(&values.new_value);
 
       values.property_name = "accessible-actions";
 
@@ -1016,13 +1034,13 @@ signal_emit_handler (gpointer p)
       gint newValue = get_int_value(jniEnv,
                                     (*jniEnv)->GetObjectArrayElement(jniEnv, args, 0));
 
-      gint prevCount = *(gint*)g_hash_table_lookup(jaw_obj->storedData,
-                                                   "Previous_Count");
+      gint prevCount = GPOINTER_TO_INT(g_hash_table_lookup(jaw_obj->storedData,
+                                                           "Previous_Count"));
       gint curCount = atk_text_get_character_count(ATK_TEXT(jaw_obj));
 
       g_hash_table_insert(jaw_obj->storedData,
                           "Previous_Count",
-                          (gpointer)&curCount);
+                          GINT_TO_POINTER(curCount));
 
       if (curCount > prevCount)
       {
@@ -1041,7 +1059,7 @@ signal_emit_handler (gpointer p)
       break;
     }
     default:
-    break;
+      break;
   }
   free_callback_para(para);
   return FALSE;
@@ -1054,14 +1072,11 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_emitSignal(JNIEnv *jniEnv,
                                                            jint id,
                                                            jobjectArray args)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
   jobjectArray global_args = (jobjectArray)(*jniEnv)->NewGlobalRef(jniEnv, args);
-  CallbackPara *para = alloc_callback_para(lock);
+  CallbackPara *para = alloc_callback_para(global_ac);
   para->signal_id = (gint)id;
   para->args = global_args;
-
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
   g_idle_add(signal_emit_handler, para);
 }
 
@@ -1074,14 +1089,16 @@ object_state_change_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** object_state_change_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("object_state_change_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** object_state_change_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("object_state_change_handler: global_ac");
     free_callback_para(para);
     return FALSE;
   }
@@ -1089,7 +1106,8 @@ object_state_change_handler (gpointer p)
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** object_state_change_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("object_state_change_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -1110,8 +1128,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_objectStateChange(JNIEnv *jniEnv
                                                                   jobject state,
                                                                   jboolean value)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   AtkStateType state_type = jaw_util_get_atk_state_type_from_java_state( jniEnv, state );
   para->atk_state = state_type;
   if (value == JNI_TRUE) {
@@ -1119,8 +1137,6 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_objectStateChange(JNIEnv *jniEnv
   } else {
     para->state_value = FALSE;
   }
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
   g_idle_add(object_state_change_handler, para);
 }
 
@@ -1135,14 +1151,16 @@ component_added_handler (gpointer p)
   jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** component_added_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_added_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** component_added_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_added_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -1150,7 +1168,8 @@ component_added_handler (gpointer p)
   JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** component_added_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_added_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -1165,7 +1184,6 @@ component_added_handler (gpointer p)
   }
 
   free_callback_para(para);
-
   return FALSE;
 }
 
@@ -1174,10 +1192,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentAdded(JNIEnv *jniEnv,
                                                                jclass jClass,
                                                                jobject jAccContext)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(component_added_handler, para);
 }
 
@@ -1191,22 +1207,25 @@ component_removed_handler (gpointer p)
   jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** component_removed_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_removed_handler: env == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
   if (global_ac == NULL)
   {
-    g_warning("\n *** component_removed_handler: global_ac == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_removed_handler: global_ac == NULL");
     free_callback_para(para);
     return FALSE;
   }
 
-  JawImpl* jaw_impl = jaw_impl_find_instance(jniEnv, global_ac);
+  JawImpl* jaw_impl = jaw_impl_get_instance(jniEnv, global_ac);
   if (jaw_impl == NULL)
   {
-    g_warning("\n *** component_removed_handler: jaw_impl == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_removed_handler: jaw_impl == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -1215,7 +1234,8 @@ component_removed_handler (gpointer p)
 
   if (atk_obj == NULL)
   {
-    g_warning("\n *** component_removed_handler: atk_obj == NULL *** \n");
+    if (jaw_debug)
+      g_warning("component_removed_handler: atk_obj == NULL");
     free_callback_para(para);
     return FALSE;
   }
@@ -1231,10 +1251,8 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_componentRemoved(JNIEnv *jniEnv,
                                                                  jclass jClass,
                                                                  jobject jAccContext)
 {
-  lock = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
-  CallbackPara *para = alloc_callback_para(lock);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
+  jobject global_ac = (*jniEnv)->NewGlobalRef(jniEnv, jAccContext);
+  CallbackPara *para = alloc_callback_para(global_ac);
   g_idle_add(component_removed_handler, para);
 }
 
@@ -1248,7 +1266,8 @@ key_dispatch_handler (gpointer p)
   JNIEnv *jniEnv = jaw_util_get_jni_env();
   if (jniEnv == NULL)
   {
-    g_warning("\n *** key_dispatch_handler: env == NULL *** \n");
+    if (jaw_debug)
+      g_warning("key_dispatch_handler: env == NULL");
     return FALSE;
   }
 
@@ -1352,8 +1371,6 @@ JNICALL Java_org_GNOME_Accessibility_AtkWrapper_dispatchKeyEvent(JNIEnv *jniEnv,
 {
   jboolean key_consumed;
   jobject global_key_event = (*jniEnv)->NewGlobalRef(jniEnv, jAtkKeyEvent);
-  gdk_threads_set_lock_functions(jni_threads_lock, jni_threads_unlock);
-  gdk_threads_init();
   g_idle_add(key_dispatch_handler, (gpointer)global_key_event);
 
   if(jaw_debug)
